#!/usr/bin/env node
"use strict";
var minimist = require('minimist');
var wordwrap = require('wordwrap');
var path_1 = require('path');
var updateNotifier = require('update-notifier');
var extend = require('xtend');
var events_1 = require('events');
var cli_1 = require('./support/cli');
var bundle = require('./bin-bundle');
var init = require('./bin-init');
var uninstall = require('./bin-uninstall');
var install = require('./bin-install');
var list = require('./bin-list');
var search = require('./bin-search');
var open = require('./bin-open');
var view = require('./bin-view');
var pkg = require('../package.json');
var IS_PRODUCTION = process.env.NODE_ENV === 'production';
var ALIASES = {
    i: install,
    in: install,
    install: install,
    r: uninstall,
    rm: uninstall,
    remove: uninstall,
    uninstall: uninstall,
    init: init,
    ls: list,
    ll: list,
    la: list,
    list: list,
    bundle: bundle,
    search: search,
    open: open,
    view: view,
    info: view
};
var argv = minimist(process.argv.slice(2), {
    boolean: ['version', 'save', 'saveDev', 'savePeer', 'ambient', 'verbose', 'dev', 'production'],
    string: ['cwd', 'out', 'name'],
    alias: {
        ambient: ['A'],
        version: ['v'],
        save: ['S'],
        saveDev: ['save-dev', 'D'],
        savePeer: ['savePeer', 'P'],
        verbose: ['V'],
        out: ['o'],
        help: ['h']
    }
});
var cwd = process.cwd();
var emitter = new events_1.EventEmitter();
var isDev = IS_PRODUCTION ? argv.dev : !argv.production;
var args = extend({ cwd: cwd, emitter: emitter }, argv, { dev: isDev, production: !isDev });
updateNotifier({ pkg: pkg }).notify();
exec(args);
var references = [];
emitter.on('reference', function (_a) {
    var reference = _a.reference, name = _a.name;
    if (references.indexOf(reference) === -1) {
        cli_1.logInfo("Stripped reference \"" + reference + "\" during installation of \"" + name + "\"", 'reference');
        references.push(reference);
    }
});
emitter.on('enoent', function (_a) {
    var path = _a.path;
    cli_1.logWarning("Path \"" + path + "\" is missing", 'ENOENT');
});
function exec(options) {
    if (options._.length) {
        var command = ALIASES[options._[0]];
        var args_1 = options._.slice(1);
        if (command != null) {
            if (options.help) {
                return cli_1.handle(command.help(args_1, options), options);
            }
            return cli_1.handle(command.exec(args_1, options), options);
        }
    }
    else if (options.version) {
        console.log(pkg.version);
        return;
    }
    var wrap = wordwrap(4, 80);
    console.log("\nUsage: typings <command>\n\nCommands:\n" + wrap(Object.keys(ALIASES).sort().join(', ')) + "\n\ntypings <command> -h   Get help for <command>\ntypings <command> -V   Enable verbose logging\n\ntypings --version      Print the CLI version\n\ntypings@" + pkg.version + " " + path_1.join(__dirname, '..') + "\n");
}
//# sourceMappingURL=bin.js.map